/*
 * setleds.c - aeb, 940130, 940909
 *
 * Call: setleds [-L] [-D] [-F] [{+|-}{num|caps|scroll}]*
 * will set or clear the indicated flags on the stdin tty,
 * and report the settings before and after.
 * In particular, setleds without arguments will only report.
 */

#include <stdio.h>
#include <fcntl.h>
#include <linux/kd.h>
#include <sys/ioctl.h>


#define onoff(a) ((a) ? "on " : "off")

struct led {
    char *name;
    int bit;
} leds[3] = {
    { "scroll", LED_SCR },
    { "num", LED_NUM },
    { "caps", LED_CAP }
};

int
main(argc,argv) int argc; char **argv;
{
    int optL = 0, optD = 0, optF = 0;
    char oleds, nleds, oflags, nflags, odefflags, ndefflags;
    char nval, ndef, sign;
    char *ap;
    struct led *lp;

    if (ioctl(0, KDGETLED, &oleds)) {
	perror("KDGETLED");
	fprintf(stderr,
	  "Error. not a VT?\n");
	exit(1);
    }

    if (ioctl(0, KDGKBLED, &oflags)) {
	perror("KDGKBLED");
	fprintf(stderr,
          "Error. old kernel?\n");
	exit(1);
    }

    while (argc > 1) {
	if (!strcmp("-L", argv[1]))
	  optL = 1;
	else if (!strcmp("-D", argv[1]))
	  optD = 1;
	else if (!strcmp("-F", argv[1]))
	  optF = 1;
	else 
	  break;
	argc--;
	argv++;
    }

    odefflags = ndefflags = ((oflags >> 4) & 7);
    oflags = nflags = (oflags & 7);

    if (argc <= 1) {
	if (optL) {
	    nleds = 0xff;
	    if (ioctl(0, KDSETLED, &nleds)) {
		perror("KDSETLED");
		fprintf(stderr, "Error resetting ledmode\n");
		exit(1);
	    }
	}
	exit(0);
    }

    if (!optL)
      optF = 1;
    nval = 0;
    ndef = 0;

    while(--argc) {
	ap = *++argv;
	sign = 1;		/* by default: set */
	if(*ap == '+')
	  ap++;
	else if(*ap == '-') {
	    sign = 0;
	    ap++;
	}
	for (lp = leds; lp-leds < 3; lp++) {
	    if(!strcmp(ap, lp->name)) {
		if(sign)
		  nval |= lp->bit;
		ndef |= lp->bit;
		goto nxtarg;
	    }
	}
      nxtarg: ;
    }

    if (optD) {
	ndefflags = (odefflags & ~ndef) | nval;
    }
    if (optF) {
	nflags = ((oflags & ~ndef) | nval);
    }
    if (optD || optF) {
	if (ioctl(0, KDSKBLED, (ndefflags << 4) | nflags)) {
	    perror("KDSKBLED");
	    exit(1);
	}
    }
    if (optL) {
	nleds = (oleds & ~ndef) | nval;
	if (ioctl(0, KDSETLED, nleds)) {
	    perror("KDSETLED");
	    exit(1);
	}
    }
    exit(0);
}
